"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = dbSchemaToType;
const fs_1 = __importDefault(require("fs"));
const grab_dir_names_1 = __importDefault(require("../../utils/backend/names/grab-dir-names"));
const lodash_1 = __importDefault(require("lodash"));
const ejson_1 = __importDefault(require("../../utils/ejson"));
const generate_type_definitions_1 = __importDefault(require("./generate-type-definitions"));
const path_1 = __importDefault(require("path"));
function dbSchemaToType(params) {
    var _a;
    let datasquirelSchema;
    const defaultTableFieldsJSONFilePath = path_1.default.resolve(__dirname, "../../data/defaultFields.json");
    if (params === null || params === void 0 ? void 0 : params.dbSchema) {
        datasquirelSchema = params.dbSchema;
    }
    else {
        const { mainShemaJSONFilePath } = (0, grab_dir_names_1.default)();
        const mainSchema = ejson_1.default.parse(fs_1.default.readFileSync(mainShemaJSONFilePath, "utf-8"));
        datasquirelSchema = mainSchema.find((sch) => sch.dbFullName == "datasquirel");
    }
    if (!datasquirelSchema)
        return;
    let tableNames = `export const DsqlTables = [\n${datasquirelSchema.tables
        .map((tbl) => `    "${tbl.tableName}",`)
        .join("\n")}\n] as const`;
    const defaultFields = ejson_1.default.parse(fs_1.default.readFileSync(defaultTableFieldsJSONFilePath, "utf-8"));
    const dbTablesSchemas = datasquirelSchema.tables.map((tblSchm) => {
        let newDefaultFields = lodash_1.default.cloneDeep(defaultFields);
        return Object.assign(Object.assign({}, tblSchm), { fields: (params === null || params === void 0 ? void 0 : params.dbSchema)
                ? tblSchm.fields
                : [
                    newDefaultFields.shift(),
                    newDefaultFields.shift(),
                    ...tblSchm.fields,
                    ...newDefaultFields,
                ] });
    });
    const defDbName = (datasquirelSchema.dbName ||
        ((_a = datasquirelSchema.dbFullName) === null || _a === void 0 ? void 0 : _a.replace(/datasquirel_user_\d+_/, "")))
        .toUpperCase()
        .replace(/ /g, "_");
    const schemas = dbTablesSchemas
        .map((table) => (0, generate_type_definitions_1.default)({
        paradigm: "TypeScript",
        table,
        typeDefName: `DSQL_${defDbName}_${table.tableName.toUpperCase()}`,
        allValuesOptional: true,
        addExport: true,
    }))
        .filter((schm) => typeof schm == "string");
    return [tableNames, ...schemas];
}
