import { DSQL_TableSchemaType } from "../../types";
import defaultFieldsRegexp from "./default-fields-regexp";

type Param = {
    paradigm: "JavaScript" | "TypeScript" | undefined;
    table: DSQL_TableSchemaType;
    query?: any;
    typeDefName?: string;
    allValuesOptional?: boolean;
    addExport?: boolean;
};

export default function generateTypeDefinition({
    paradigm,
    table,
    query,
    typeDefName,
    allValuesOptional,
    addExport,
}: Param): string | null {
    let typeDefinition: string | null = ``;

    try {
        const tdName =
            typeDefName ||
            `DSQL_${query.single}_${query.single_table}`.toUpperCase();

        const fields = table.fields;

        function typeMap(type: string) {
            if (type?.match(/int/i)) {
                return "number";
            }
            if (type?.match(/text|varchar|timestamp/i)) {
                return "string";
            }

            return "string";
        }

        const typesArrayTypeScript = [];
        const typesArrayJavascript = [];

        typesArrayTypeScript.push(
            `${addExport ? "export " : ""}type ${tdName} = {`
        );
        typesArrayJavascript.push(`/**\n * @typedef {object} ${tdName}`);

        fields.forEach((field) => {
            const nullValue = allValuesOptional
                ? "?"
                : field.nullValue
                ? "?"
                : field.fieldName?.match(defaultFieldsRegexp)
                ? "?"
                : "";

            typesArrayTypeScript.push(
                `    ${field.fieldName}${nullValue}: ${typeMap(
                    field.dataType || ""
                )};`
            );

            typesArrayJavascript.push(
                ` * @property {${typeMap(field.dataType || "")}${nullValue}} ${
                    field.fieldName
                }`
            );
        });

        typesArrayTypeScript.push(`}`);
        typesArrayJavascript.push(` */`);

        if (paradigm?.match(/javascript/i)) {
            typeDefinition = typesArrayJavascript.join("\n");
        }

        if (paradigm?.match(/typescript/i)) {
            typeDefinition = typesArrayTypeScript.join("\n");
        }
    } catch (error: any) {
        console.log(error.message);
        typeDefinition = null;
    }

    return typeDefinition;
}
